# -*- coding: utf-8 -*-

from mekk.xmind import XMindDocument, XMIND_MARKS
from nozbe import NozbeLoader
import simplejson
from twisted.internet import defer

# Znaczniki używane do kontekstów
NEXT_ACTION_MARK = 'task-start'
DONE_MARK = 'task-done'

# Ustalone mapowania
CONTEXT_MARKS = {
    "icon-0.png" : "other-exclam",
    "icon-9.png" : "flag-red",
    "icon-11.png" : "task-pause",
    "icon-14.png" : "other-lightbulb",
    "icon-16.png" : "other-people",
    "icon-43.png" : "other-phone",
    "icon-44.png" : "other-email",
    "icon-47.png" : "other-fax", # computer ...
    "icon-51.png" : "other-coffee-cup", 
    "icon-52.png" : "other-clock",
    "icon-53.png" : "other-question",
}  

class MarkerGen(object):
    """
    Klasa przypisująca ikonom kontekstów z Nozbe markery dla XMinda
    """
    def __init__(self):
        self.spare = [ x for x in XMIND_MARKS if not (
            x in CONTEXT_MARKS.values() or x in [NEXT_ACTION_MARK, DONE_MARK]) ]
        self.spare.reverse()
        self.fixed = dict()
    def give_mark(self, icon):
        if icon is None or icon == "icon-.png":
            return None
        f = self.fixed.get(icon, None)
        if f:
            return f
        f = CONTEXT_MARKS.get(icon, None)
        if f:
            return f
        if self.spare:
            f = self.spare.pop()
        else:
            f = None
        self.fixed[icon] = f
        return f

@defer.inlineCallbacks
def nozbe_to_xmind(loader, xmind_filename):
    """
    Podłącza się do zadanego konta XMind, pobiera projekty,
    konteksty i akcje i wszystko zapisuje
    
    @param loader: obiekt ładujący (zwykle NozbeLoader(api_key) ale można do testów inne)
    @param xmind_filename: tworzony plik .xmind
    """
    all_actions = []
    
    xmw = XMindDocument.create("Projects", "Projects")
    sheet = xmw.get_first_sheet()
    project_root = sheet.get_root_topic() 

    marker_gen = MarkerGen()
    
    projects = yield loader.get_projects()
    project_topics = dict()       # id projektu -> Topic
    
    for p in projects:
        p_id = p['id']
        p_topic = project_root.add_subtopic(p['name'], p_id)
        p_topic.set_link("http://www.nozbe.com/account/projects/show-" + p_id)

        actions = yield loader.get_project_actions(p_id)
        for action in actions:
            # Każda akcja ma 'id', 'name', 'name_show', 'done', 'time', 'project_id',
            # 'project_name', 'context_id', 'context_name', 'context_icon', 'next'
            a_topic = p_topic.add_subtopic(action['name'], action['id'])
            if action['next']:
                a_topic.add_marker(NEXT_ACTION_MARK)
            if action['done']:
                a_topic.add_marker(DONE_MARK)
            c_mark = marker_gen.give_mark(action['context_icon'])
            if c_mark:
                a_topic.add_marker(c_mark)

        all_actions.extend(actions)

    # Legenda
    legend_root = project_root.add_subtopic("Symbols", detached = True)
    common_root = legend_root.add_subtopic("Common")
    common_root.add_subtopic("Next action").add_marker(NEXT_ACTION_MARK)
    common_root.add_subtopic("Done").add_marker(DONE_MARK)

    contexts = yield loader.get_contexts()
    if contexts:
        context_root = legend_root.add_subtopic("Contexts")
        for c in contexts:
            c_id = c["id"]
            c_topic = context_root.add_subtopic(c["name"], c_id)
            c_topic.set_link("http://www.nozbe.com/account/contexts/show-" + c_id)
            c_mark = marker_gen.give_mark(c["icon"])
            if c_mark:
                c_topic.add_marker(c_mark)

    data_root = legend_root.add_subtopic(
        "Do not remove", 
        "ffffffffffffff",
        detached = True)
    data_root.set_note("""Attached file contains data necessary
to process edited mindmap. Do not open, do not remove, do not touch.
But you can move this topic to the bottom or where you like""")
    data_root.set_attachment(
        simplejson.dumps(dict(projects = projects, contexts = contexts, actions = all_actions)),
        ".json")
    
    xmw.save(xmind_filename)

